@@ -97,7 +97,7 @@ def memoize(obj):
 _port_lock = threading.RLock()
 
 def get_port():
-    """Thread safely returns a round-robbed port in the range ``g5k_configuration['port_range']``"""
+    """Thread safely returns a round-robbed port in the range ``configuration['port_range']``"""
     with _port_lock:
         if not hasattr(get_port, "current"):
             get_port.current = random.randrange(configuration['port_range'][0], configuration['port_range'][1])
@@ -106,3 +106,10 @@ def get_port():
             if get_port.current >= configuration['port_range'][1]:
                 get_port.current = configuration['port_range'][0]
         return get_port.current
+
+def singleton_to_collection(arg):
+    """Converts single object (including strings) to list of length one containing it"""
+    if not hasattr(arg, "__iter__"):
+        return [ arg ]
+    else:
+        return arg
