@@ -22,9 +22,10 @@ from execo.action import Remote, ActionNotificationProcessLH, \
 from execo.config import make_connection_params
 from execo.host import get_hosts_set, Host
 from execo.log import style, logger
-from execo.process import ProcessOutputHandler, get_process
+from execo.process import ProcessOutputHandler, get_process, \
+    handle_process_output
 from execo.time_utils import format_seconds
-from execo.utils import comma_join, compact_output
+from execo.utils import comma_join, compact_output, singleton_to_collection
 from execo_g5k.config import default_frontend_connection_params
 from execo_g5k.utils import get_frontend_host, get_kavlan_host_name
 from utils import get_default_frontend
@@ -106,14 +107,8 @@ class _KadeployStdoutHandler(ProcessOutputHandler):
 
     """Parse kadeploy3 stdout."""
 
-    def __init__(self, kadeployer, frontend):
-        """
-        :param kadeployer: the `execo_g5k.kadeploy.Kadeployer` to
-          which this `execo.process.ProcessOutputHandler` is attached.
-        """
+    def __init__(self):
         super(_KadeployStdoutHandler, self).__init__()
-        self.kadeployer = kadeployer
-        self.frontend = frontend
         self._SECTION_NONE, self._SECTION_DEPLOYED_NODES, self._SECTION_UNDEPLOYED_NODES = range(3)
         self._current_section = self._SECTION_NONE
 
@@ -121,8 +116,6 @@ class _KadeployStdoutHandler(ProcessOutputHandler):
         self._current_section = self._SECTION_NONE
 
     def read_line(self, process, string, eof, error):
-        if self.kadeployer.out:
-            print str(self.frontend) + ": " + string,
         if (_ksoh_deployed_nodes_header_re1.search(string) != None
             or _ksoh_deployed_nodes_header_re2.search(string) != None):
             self._current_section = self._SECTION_DEPLOYED_NODES
@@ -135,31 +128,14 @@ class _KadeployStdoutHandler(ProcessOutputHandler):
             so = _ksoh_deployed_node_re.search(string)
             if so != None:
                 host_address = so.group(1)
-                self.kadeployer.deployed_hosts.add(host_address)
-                self.kadeployer._frontend_processes[process]["deployed_hosts"].add(host_address)
+                process.kadeployer.deployed_hosts.add(host_address)
+                process.deployed_hosts.add(host_address)
         elif self._current_section == self._SECTION_UNDEPLOYED_NODES:
             so = _ksoh_undeployed_node_re.search(string)
             if so != None:
                 host_address = so.group(1)
-                self.kadeployer.undeployed_hosts.add(host_address)
-                self.kadeployer._frontend_processes[process]["undeployed_hosts"].add(host_address)
-
-class _KadeployStderrHandler(ProcessOutputHandler):
-
-    """Parse kadeploy3 stderr."""
-
-    def __init__(self, kadeployer, frontend):
-        """
-        :param kadeployer: the `execo_g5k.kadeploy.Kadeployer` to
-          which this `execo.process.ProcessOutputHandler` is attached.
-        """
-        super(_KadeployStderrHandler, self).__init__()
-        self.kadeployer = kadeployer
-        self.frontend = frontend
-
-    def read_line(self, process, string, eof, error):
-        if self.kadeployer.out:
-            print str(self.frontend) + ": " + string,
+                process.kadeployer.undeployed_hosts.add(host_address)
+                process.undeployed_hosts.add(host_address)
 
 _host_site_re1 = re.compile("^[^ \t\n\r\f\v\.]+\.([^ \t\n\r\f\v\.]+)\.grid5000.fr$")
 _host_site_re2 = re.compile("^[^ \t\n\r\f\v\.]+\.([^ \t\n\r\f\v\.]+)$")
@@ -180,7 +156,7 @@ def _get_host_frontend(host):
     #   extract site from host name
     #
     # - raises exception if host name format invalid (not sure we
-    #   would want this for a generic function
+    #   would want this for a generic function)
     frontend = None
     mo1 = _host_site_re1.search(host.address)
     if mo1 != None:
@@ -197,6 +173,18 @@ def _get_host_frontend(host):
                 raise ValueError, "unknown frontend for host %s" % host.address
     return frontend
 
+class FrontendPrefixWrapper(ProcessOutputHandler):
+
+    def __init__(self, handler):
+        super(FrontendPrefixWrapper, self).__init__()
+        self.handler = handler
+
+    def read_line(self, process, string, eof, error):
+        handle_process_output(process,
+                              self.handler,
+                              process.frontend + ": " + string,
+                              eof, error)
+
 class Kadeployer(Remote):
 
     """Deploy an environment with kadeploy3 on several hosts.
@@ -204,7 +192,8 @@ class Kadeployer(Remote):
     Able to deploy in parallel to multiple frontends.
     """
 
-    def __init__(self, deployment, frontend_connection_params = None):
+    def __init__(self, deployment, frontend_connection_params = None,
+                 stdout_handlers = None, stderr_handlers = None):
         """
         :param deployment: instance of Deployment class describing the
           intended kadeployment.
@@ -212,6 +201,12 @@ class Kadeployer(Remote):
         :param frontend_connection_params: connection params for
           connecting to frontends if needed. Values override those in
           `execo_g5k.config.default_frontend_connection_params`.
+
+        :param stdout_handlers: iterable of `ProcessOutputHandlers`
+          which will be passed to the actual deploy processes.
+
+        :param stderr_handlers: iterable of `ProcessOutputHandlers`
+          which will be passed to the actual deploy processes.
         """
         super(Remote, self).__init__()
         self.deployed_hosts = set()
@@ -220,8 +215,10 @@ class Kadeployer(Remote):
         self.undeployed_hosts = set()
         """Iterable of `Host` containing the hosts not deployed. This iterable
         won't be complete if the Kadeployer has not terminated."""
-        self.out = False
-        """If True, output kadeploy stdout / stderr to stdout."""
+        self._stdout_handlers = stdout_handlers
+        self._stderr_handlers = stderr_handlers
+        """Iterable of `ProcessOutputHandlers` which will be passed to the
+        actual deploy processes."""
         self.frontend_connection_params = frontend_connection_params
         """Connection params for connecting to frontends if needed. Values override
         those in `execo_g5k.config.default_frontend_connection_params`."""
@@ -234,7 +231,6 @@ class Kadeployer(Remote):
 
     def _init_processes(self):
         self.processes = []
-        self._frontend_processes = {}
         self._unique_hosts = get_hosts_set(self.deployment.hosts)
         frontends = dict()
         for host in self._unique_hosts:
@@ -244,6 +240,7 @@ class Kadeployer(Remote):
             else:
                 frontends[frontend] = [host]
         lifecycle_handler = ActionNotificationProcessLH(self, len(frontends))
+        deploy_stdout_handler = _KadeployStdoutHandler()
         for frontend in frontends.keys():
             kadeploy_command = self.deployment._get_common_kadeploy_command_line()
             for host in frontends[frontend]:
@@ -254,44 +251,44 @@ class Kadeployer(Remote):
                                                                      default_frontend_connection_params))
             p.pty = True
             p.timeout = self.timeout
-            kdstdouthandler = _KadeployStdoutHandler(self, frontend)
-            p.stdout_handlers.append(kdstdouthandler)
-            kdstderrhandler = _KadeployStderrHandler(self, frontend)
-            p.stderr_handlers.append(kdstderrhandler)
+            p.stdout_handlers.append(deploy_stdout_handler)
+            p.stdout_handlers.extend([ FrontendPrefixWrapper(h)
+                                       for h in singleton_to_collection(self._stdout_handlers) ])
+            p.stderr_handlers.extend([ FrontendPrefixWrapper(h)
+                                       for h in singleton_to_collection(self._stderr_handlers) ])
             p.lifecycle_handlers.append(lifecycle_handler)
+            p.frontend = frontend
+            p.kadeploy_hosts = [ host.address for host in frontends[frontend] ]
+            p.deployed_hosts = set()
+            p.undeployed_hosts = set()
+            p.kadeployer = self
             self.processes.append(p)
-            self._frontend_processes[p] = {}
-            self._frontend_processes[p]["frontend"] = frontend
-            self._frontend_processes[p]["hosts"] = [host.address for host in frontends[frontend]]
-            self._frontend_processes[p]["deployed_hosts"] = set()
-            self._frontend_processes[p]["undeployed_hosts"] = set()
 
     def _common_reset(self):
         super(Kadeployer, self)._common_reset()
         self.deployed_hosts = set()
         self.undeployed_hosts = set()
 
-
     def _check_ok(self, log):
         ok = True
         error_logs = []
         warn_logs = []
-        for fprocess, hosts_lists in self._frontend_processes.iteritems():
-            if ( len(hosts_lists["deployed_hosts"].intersection(hosts_lists["undeployed_hosts"])) != 0
-                 or len(hosts_lists["deployed_hosts"].union(hosts_lists["undeployed_hosts"]).symmetric_difference(hosts_lists["hosts"])) != 0 ):
+        for process in self.processes:
+            if ( len(process.deployed_hosts.intersection(process.undeployed_hosts)) != 0
+                 or len(process.deployed_hosts.union(process.undeployed_hosts).symmetric_difference(process.kadeploy_hosts)) != 0 ):
                 error_logs.append("deploy on %s, total/deployed/undeployed = %i/%i/%i:\n%s\nstdout:\n%s\nstderr:\n%s" % (
-                        hosts_lists["frontend"], len(hosts_lists["hosts"]), len(hosts_lists["deployed_hosts"]), len(hosts_lists["undeployed_hosts"]),
-                        fprocess, compact_output(fprocess.stdout), compact_output(fprocess.stderr)))
+                        process.frontend, len(process.kadeploy_hosts), len(process.deployed_hosts), len(process.undeployed_hosts),
+                        process, compact_output(process.stdout), compact_output(process.stderr)))
                 ok = False
             else:
-                if len(hosts_lists["deployed_hosts"]) == 0:
+                if len(process.deployed_hosts) == 0:
                     warn_logs.append("deploy on %s, total/deployed/undeployed = %i/%i/%i:\n%s\nstdout:\n%s\nstderr:\n%s" % (
-                            hosts_lists["frontend"], len(hosts_lists["hosts"]), len(hosts_lists["deployed_hosts"]), len(hosts_lists["undeployed_hosts"]),
-                            fprocess, compact_output(fprocess.stdout), compact_output(fprocess.stderr)))
+                            process.frontend, len(process.kadeploy_hosts), len(process.deployed_hosts), len(process.undeployed_hosts),
+                            process, compact_output(process.stdout), compact_output(process.stderr)))
                     ok = False
-                elif len(hosts_lists["undeployed_hosts"]) > 0:
+                elif len(process.undeployed_hosts) > 0:
                     warn_logs.append("deploy on %s, total/deployed/undeployed = %i/%i/%i" % (
-                            hosts_lists["frontend"], len(hosts_lists["hosts"]), len(hosts_lists["deployed_hosts"]), len(hosts_lists["undeployed_hosts"])))
+                            process.frontend, len(process.kadeploy_hosts), len(process.deployed_hosts), len(process.undeployed_hosts)))
         if log:
             if len(warn_logs) > 0:
                 logger.warn(str(self) + ":\n" + "\n".join(warn_logs))
@@ -327,7 +324,8 @@ def deploy(deployment,
            frontend_connection_params = None,
            deploy_timeout = None,
            check_timeout = 30,
-           out = False):
+           stdout_handlers = None,
+           stderr_handlers = None):
     """Deploy nodes, many times if needed, checking which of these nodes are already deployed with a user-supplied command. If no command given for checking if nodes deployed, rely on kadeploy to know which nodes are deployed.
 
     - loop `num_tries` times:
@@ -389,7 +387,11 @@ def deploy(deployment,
     :param check_timeout: timeout for node deployment checks. Default
       is 30 seconds.
 
-    :param out: if True, output kadeploy stdout / stderr to stdout.
+    :param stdout_handlers: iterable of `ProcessOutputHandlers`
+          which will be passed to the actual deploy processes.
+
+    :param stderr_handlers: iterable of `ProcessOutputHandlers`
+          which will be passed to the actual deploy processes.
     """
 
     if check_enough_func == None:
@@ -454,8 +456,9 @@ def deploy(deployment,
         tmp_deployment = copy.copy(deployment)
         tmp_deployment.hosts = undeployed_hosts
         kadeployer = Kadeployer(tmp_deployment,
-                                frontend_connection_params = frontend_connection_params)
-        kadeployer.out = out
+                                frontend_connection_params = frontend_connection_params,
+                                stdout_handlers = stdout_handlers,
+                                stderr_handlers = stderr_handlers)
         kadeployer.timeout = deploy_timeout
         kadeployer.run()
         my_newly_deployed = []
